@@ -5,7 +5,7 @@ import time
 import warnings
 from threading import local
 
-from neo4j import GraphDatabase, basic_auth
+from neo4j import GraphDatabase, basic_auth, DEFAULT_DATABASE
 from neo4j.exceptions import ClientError, SessionExpired
 
 from neo4j.graph import Node
@@ -14,8 +14,9 @@ from . import config
 from .exceptions import UniqueProperty, ConstraintValidationFailed, ModelDefinitionMismatch
 
 if sys.version_info >= (3, 0):
-    from urllib.parse import urlparse
+    from urllib.parse import quote, unquote, urlparse
 else:
+    from urllib import quote, unquote  # noqa
     from urlparse import urlparse  # noqa
 
 logger = logging.getLogger(__name__)
@@ -82,28 +83,50 @@ class Database(local, NodeClassRegistry):
         self.url = None
         self.driver = None
         self._pid = None
+        self._database_name = DEFAULT_DATABASE
 
     def set_connection(self, url):
         """
         Sets the connection URL to the address a Neo4j server is set up at
         """
+        p_start = url.replace(':', '', 1).find(':') + 2
+        p_end = url.rfind('@')
+        password = url[p_start:p_end]
+        url = url.replace(password, quote(password))
         u = urlparse(url)
 
         valid_schemas = ['bolt', 'bolt+s', 'bolt+ssc', 'bolt+routing', 'neo4j', 'neo4j+s', 'neo4j+ssc']
 
         if u.netloc.find('@') > -1 and u.scheme in valid_schemas:
             credentials, hostname = u.netloc.rsplit('@', 1)
-            username, password, = credentials.split(':')
+            username, password = credentials.split(':')
+            password = unquote(password)
+            database_name = u.path.strip("/")
         else:
             raise ValueError("Expecting url format: bolt://user:password@localhost:7687"
                              " got {0}".format(url))
 
-        self.driver = GraphDatabase.driver(u.scheme + '://' + hostname,
-                                           auth=basic_auth(username, password),
-                                           max_connection_pool_size=config.MAX_CONNECTION_POOL_SIZE)
+        options = dict(
+           auth=basic_auth(username, password),
+           connection_acquisition_timeout=config.CONNECTION_ACQUISITION_TIMEOUT,
+           connection_timeout=config.CONNECTION_TIMEOUT,
+           keep_alive=config.KEEP_ALIVE,
+           max_connection_lifetime=config.MAX_CONNECTION_LIFETIME,
+           max_connection_pool_size=config.MAX_CONNECTION_POOL_SIZE,
+           max_transaction_retry_time=config.MAX_TRANSACTION_RETRY_TIME,
+           resolver=config.RESOLVER,
+           user_agent=config.USER_AGENT
+        )
+
+        if "+s" not in u.scheme:
+            options['encrypted'] = config.ENCRYPTED
+            options['trust'] = config.TRUST
+
+        self.driver = GraphDatabase.driver(u.scheme + '://' + hostname, **options)
         self.url = url
         self._pid = os.getpid()
         self._active_transaction = None
+        self._database_name = DEFAULT_DATABASE if database_name == "" else database_name
 
     @property
     def transaction(self):
@@ -127,7 +150,7 @@ class Database(local, NodeClassRegistry):
         """
         if self._active_transaction:
             raise SystemError("Transaction in progress")
-        self._active_transaction = self.driver.session(default_access_mode=access_mode).begin_transaction()
+        self._active_transaction = self.driver.session(default_access_mode=access_mode, database=self._database_name).begin_transaction()
 
     @ensure_connection
     def commit(self):
@@ -210,7 +233,7 @@ class Database(local, NodeClassRegistry):
         if self._active_transaction:
             session = self._active_transaction
         else:
-            session = self.driver.session()
+            session = self.driver.session(database=self._database_name)
 
         try:
             # Retrieve the data
@@ -244,8 +267,9 @@ class Database(local, NodeClassRegistry):
                                          retry_on_session_expire=False)
             raise
 
-        if os.environ.get('NEOMODEL_CYPHER_DEBUG', False):
-            logger.debug("query: " + query + "\nparams: " + repr(params) + "\ntook: {:.2g}s\n".format(end - start))
+        tte = (end - start)
+        if os.environ.get('NEOMODEL_CYPHER_DEBUG', False) and tte > float(os.environ.get('NEOMODEL_SLOW_QUERIES', 0)):
+            logger.debug("query: " + query + "\nparams: " + repr(params) + "\ntook: {:.2g}s\n".format(tte))
 
         return results, meta
 
